# -*- coding: utf-8 -*-

import os

_ROOT = os.path.abspath(os.path.dirname(__file__))

def get_template_path(name):
    return os.path.join(_ROOT, 'templates', name)

def get_conan_settings_path():
    return os.path.join(_ROOT, 'conan_settings')

def get_conan_settings_yml_path():
    return os.path.join(get_conan_settings_path(), 'settings.yml')

def get_conan_profiles_path(name):
    return os.path.join(get_conan_settings_path(), 'profiles', name)

def get_setup_path():
    return os.path.abspath(os.path.join(_ROOT, os.pardir))



# -*- coding: utf-8 -*-

import argparse
import configparser
import os
import shutil
import subprocess
import platform
import re
import time

import docker
import vswhere

import sesame

from colorama import Fore, Back, Style

def build_cmd(args):
    """Build a recipe"""
    parser = argparse.ArgumentParser(description=build_cmd.__doc__, prog='sesame build')
    parser.add_argument('--upload', action='store_true', default=False)
    parser.add_argument("--user", '-u', help="Conan user that the package will be created under.", default='sesame')
    parser.add_argument("--channel", '-c', help="Conan channel that the package will be created under.", default='testing')

    parser.add_argument("--default-profile", help="Uses sesame-default.profile instead of i.e. sesame-base-windows.profile", default=False, action="store_true")
    parser.add_argument("--build-missing", help="Build missing deps", default=False, action="store_true")

    parser.add_argument("--android", help="builds for android", default=False, action="store_true")
    parser.add_argument("--emscripten", help="builds for emscripten", default=False, action="store_true")
    parser.add_argument("--linux", help="builds for linux", default=False, action="store_true")
    parser.add_argument("--macos", help="builds for macOS", default=False, action="store_true")
    parser.add_argument("--ios", help="builds for iOS", default=False, action="store_true")
    parser.add_argument("--windows", help="builds for windows", default=False, action="store_true")

    args = parser.parse_args(*args)
    _build(args)

def _build(args):
    common_conan_env = {
        'CONAN_USERNAME': f'{args.user}',
        'CONAN_CHANNEL': f'{args.channel}',

        'CONAN_PIP_PACKAGE': 'False',
        'CONAN_STABLE_BRANCH_PATTERN': ' '
    }

    upload_conan_env = {}
    if args.upload:
        upload_conan_env = {
            'CONAN_LOGIN_USERNAME': 'orhun',
            'CONAN_UPLOAD': 'https://api.bintray.com/conan/orhun/sesame',
            'CONAN_UPLOAD_ONLY_WHEN_STABLE': '0',
        }

    build_conan_env = {}
    if args.build_missing:
        build_conan_env = {
            'CONAN_BUILD_POLICY': 'missing'
        }

    platform_activity_map = {
        'android': args.android,
        'emscripten': args.emscripten,
        'linux': args.linux,
        'macos': args.macos,
        'ios': args.ios,
        'windows': args.windows
    }

    # if it is not given just build it for the host platform
    if not any(platform_activity_map.values()):
        build = {
            'Windows': 'windows',
            'Darwin': 'macos',
            'Linux': 'linux'
        }.get(platform.system())
        platform_activity_map[build] = True
        print(Fore.YELLOW + Style.BRIGHT + f'\nNo platform given. Building for {build}!')
        time.sleep(2)

    for build in platform_activity_map.items():
        name = build[0]
        active = build[1]
        if active:
            platform_conan_envs = _prepare_conan_env(args, prep_for=name)

            for platform_conan_env in platform_conan_envs:
                conan_env = {**common_conan_env, **upload_conan_env, **build_conan_env, **platform_conan_env}

                build_script = 'build.py'
                if os.path.isfile('build-sesame.py'):
                    build_script = 'build-sesame.py'
                python = {
                    'Windows': 'python',
                    'Darwin': 'python3',
                    'Linux': 'python3'
                }.get(platform.system())
                subprocess.run([python, build_script], check=True, cwd='.', env={**os.environ.copy(), **conan_env})

def _prepare_conan_env(args, prep_for):
    envs = [{}]

    with open('build-sesame.py') as file:
      if 'builder = build_template_tool.get_builder()' in file.read():
        return envs # none of the following is needed for building tools

    envs[0]['CONAN_BUILD_TYPES'] = 'Debug'
    envs[0]['CONAN_APPLE_CLANG_VERSIONS'] = '11.0'
    envs[0]['CONAN_VISUAL_VERSIONS'] = '16'
    envs[0]['CONAN_CPPSTDS'] = '17'

    if prep_for == 'android':
        envs[0]['SESAME_BUILD_FOR'] = prep_for
        envs[0]['CONAN_CLANG_VERSIONS'] = '8'
        envs[0]['CONAN_ARCHS'] = 'armv8,x86_64'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-android-28.profile')
    elif prep_for == 'emscripten':
        envs[0]['SESAME_BUILD_FOR'] = prep_for
        envs[0]['CONAN_CLANG_VERSIONS'] = '10'
        envs[0]['CONAN_ARCHS'] = 'wasm'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-emscripten.profile')
    elif prep_for == 'linux':
        envs[0]['CONAN_ARCHS'] = 'x86_64'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-linux.profile')
    elif prep_for == 'macos':
        envs[0]['CONAN_ARCHS'] = 'x86_64'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-macos.profile')
    elif prep_for == 'ios':
        envs[0]['CONAN_ARCHS'] = 'x86_64,armv8'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-ios.profile')
    elif prep_for == 'windows':
        envs[0]['CONAN_VISUAL_RUNTIMES'] = 'MTd'
        envs[0]['CONAN_ARCHS'] = 'x86_64'
        envs[0]['CONAN_BASE_PROFILE'] = sesame.get_conan_profiles_path('sesame-base-windows.profile')

    return envs



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from cpt.packager import ConanMultiPackager
from cpt.builds_generator import BuildGenerator, get_mingw_builds, get_visual_builds, get_linux_clang_builds, get_linux_gcc_builds, get_osx_apple_clang_builds

def _get_builds(self, pure_c, shared_option_name, dll_with_static_runtime, reference=None):
  ref = reference or self._reference
  if self._os_name in ["Android", "Emscripten"]:
    return get_linux_clang_builds(self._clang_versions, self._archs, shared_option_name,
                                  pure_c, self._build_types, self._cppstds, self._options, ref)
  else:
    raise Exception("Unknown operating system: %s" % self._os_name)

def get_builder():
  platform_info = None
  build_for = os.getenv('SESAME_BUILD_FOR', '')
  if build_for == 'android':
    BuildGenerator.get_builds = _get_builds
    class AndroidPlatformInfo(object):
      @staticmethod
      def system():
        return 'Android'
    platform_info = AndroidPlatformInfo()
  elif build_for == 'emscripten':
    BuildGenerator.get_builds = _get_builds
    class EmscriptenPlatformInfo(object):
      @staticmethod
      def system():
        return 'Emscripten'
    platform_info = EmscriptenPlatformInfo()

  builder = ConanMultiPackager(platform_info=platform_info)
  return builder



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import platform
from conans.client.conan_api import Conan
from sesame import build_shared

import cpt.builds_generator

def get_builder(pure_c=False):
    os_build = {
        'Windows': 'Windows',
        'Darwin': 'Macos',
        'Linux': 'Linux'}.get(platform.system())

    builder = build_shared.get_builder()
    builder.add_common_builds(pure_c=pure_c)
    builder.remove_build_if(lambda build: 'compiler.libcxx' in build.settings and build.settings['compiler.libcxx'] == 'libstdc++')
    for build_conf in builder.items:
      build_conf.settings['arch_build'] = 'x86_64'
      build_conf.settings['os_build'] = os_build

    return builder



# -*- coding: utf-8 -*-

import argparse
import configparser
import os
import shutil
import subprocess
import platform
import re

import docker
import vswhere
import psutil

import sesame

from colorama import Fore, Back, Style

def dbuild_cmd(args):
    """Build a recipe in docker"""
    parser = argparse.ArgumentParser(description=dbuild_cmd.__doc__, prog='sesame build')
    parser.add_argument("--user", '-u', help="Conan user that the package will be created under.", default='sesame')
    parser.add_argument("--channel", '-c', help="Conan channel that the package will be created under.", default='testing')

    parser.add_argument("--android", help="builds for android", default=False, action="store_true")
    parser.add_argument("--emscripten", help="builds for emscripten", default=False, action="store_true")
    parser.add_argument("--windows", help="builds for windows", default=False, action="store_true")
    parser.add_argument("--uwp", help="builds for windows", default=False, action="store_true")

    args = parser.parse_args(*args)
    _build(args)

def _is_docker_running():
    return True
    service = None
    try:
        service = psutil.win_service_get('docker')
        service = service.as_dict()
    except Exception:
        pass

    return service is not None

def _build(args):
    if args.android:
        docker_image = 'sesameorhun/android-devel'
        sesame_mount = '/tmp/sesame'
        recipe_mount = '/tmp/recipe'
        result_mount = '/tmp/result'
        conan_user_home = '~/'
        command = f'/bin/bash -c "cd {recipe_mount} && sesame build --android --user {args.user} --channel {args.channel} && conan remove */*@* --builds --src --force && cp -r ~/.conan {result_mount}"'
    elif args.emscripten:
        docker_image = 'sesameorhun/emscripten-devel'
        sesame_mount = '/tmp/sesame'
        recipe_mount = '/tmp/recipe'
        result_mount = '/tmp/result'
        conan_user_home = '~/'
        command = f'/bin/bash -c "cd {recipe_mount} && sesame build --emscripten --user {args.user} --channel {args.channel} && conan remove */* --builds --src --force && cp -r ~/.conan {result_mount}"'
    elif args.windows:
        docker_image = 'sesameorhun/windows-devel'
        sesame_mount = 'C:\\sesame'
        recipe_mount = 'C:\\recipe'
        result_mount = 'C:\\result'
        conan_user_home = 'C:\\conan'
        command = f'powershell -command cd sesame ; pip install . ; cd ..\\\\recipe ; sesame build --windows --user {args.user} --channel {args.channel} ; conan remove --builds --src --force * ; robocopy "$env:USERPROFILE\\.conan" "c:\\result" /MIR /J /MOVE'
    elif args.uwp:
        docker_image = 'sesameorhun/windows-devel'
        sesame_mount = 'C:\\sesame'
        recipe_mount = 'C:\\recipe'
        result_mount = 'C:\\result'
        conan_user_home = 'C:\\conan'
        command = f'powershell -command cd sesame ; pip install . ; cd ..\\\\recipe ; sesame build --uwp --user {args.user} --channel {args.channel} ; conan remove --builds --src --force * ; robocopy "$env:USERPROFILE\\.conan" "c:\\result" /MIR /J /MOVE'

    is_docker_running = _is_docker_running()

    if is_docker_running:
        try:
            print(Style.BRIGHT + Fore.LIGHTWHITE_EX + 'Starting docker...')

            docker_client = docker.from_env(version='auto')
            container = docker_client.containers.run(
                image=docker_image,
                command=command,
                volumes={
                    # sesame.get_setup_path(): {
                    #     'bind': sesame_mount,
                    #     'mode': 'rw'
                    # },
                    os.getcwd(): {
                        'bind': recipe_mount,
                        'mode': 'rw'
                    },
                    'G:\\Devel\\projects\\sesame2\\.build': {
                        'bind': result_mount,
                        'mode': 'rw'
                    }
                },
                environment={
                    'CONAN_USER_HOME_SHORT': 'None',
                    'CONAN_TEMP_TEST_FOLDER': 'False'
                },
                detach=True,
                auto_remove=True,
                stdout=True,
                stderr=True)

            for log in container.logs(stream=True):
                print(log.decode().rstrip('\r\n'))
        except Exception as e:
            print(Style.BRIGHT + Fore.LIGHTRED_EX + str(e))
    else:
        print(Style.BRIGHT + Fore.LIGHTRED_EX + "Docker is not running.")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import colorama
import inspect
import sys
import os
from argparse import ArgumentError, ArgumentParser
from sesame.commands.new import new_cmd
from sesame.commands.build import build_cmd
from sesame.commands.dbuild import dbuild_cmd

class Command(object):
    def __init__(self):
        pass

    def help(self, *args):
        print('Display Help')

    def new(self, *args):
        new_cmd(args)

    def build(self, *args):
        build_cmd(args)

    def dbuild(self, *args):
        dbuild_cmd(args)

    def _commands(self):
        result = {}
        for m in inspect.getmembers(self, predicate=inspect.ismethod):
            method_name = m[0]
            if not method_name.startswith('_'):
                method = m[1]
                result[method_name.replace('_', '-')] = method
        return result

    def _run(self, args):
        if args:
            commands = self._commands()
            if args[0] in commands:
                command = args[0]
                method = commands[command]
                method(args[1:])
            else:
                print("No command given.")

def _main(args):
    colorama.init(autoreset=True)
    command = Command()
    command._run(args)

def run():
    _main(sys.argv[1:])

if __name__ == '__main__':
    run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import re
from setuptools import setup, find_packages

def _load_long_description():
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
        return long_description

def _get_requires(filename):
    requirements = []
    with open(filename) as req_file:
        for line in req_file.read().splitlines():
            if not line.strip().startswith("#"):
                requirements.append(line)
    return requirements

def _my_version():
    def empty_scheme(version):
        return ''
    return {'local_scheme': empty_scheme}

setup(
    name='sesame-package-tools',
    use_scm_version=_my_version,
    setup_requires=['setuptools_scm'],
    long_description=_load_long_description(),
    description='Sesame Packaging Tools for building Conan recipes',
    url='https://github.com/birsoyo/sesame-package-tools',
    author='Orhun Birsoy',
    author_email='orhunbirsoy@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6'
    ],
    keywords=['conan', 'C/C++', 'package', 'libraries', 'developer', 'manager', 'dependency', 'tool', 'c', 'c++', 'cpp'],
    packages=find_packages(exclude=['tests']),
    install_requires=_get_requires(os.path.join('sesame', 'requirements.txt')),
    extras_require={'test': _get_requires(os.path.join('sesame', 'requirements_test.txt'))},
    python_requires='>=3.6',
    package_data={
        '': ['*.md'],
        'sesame' : ['*.txt', 'templates/*', 'conan_settings/*', 'conan_settings/profiles/*'],
    },
    entry_points={
        'console_scripts': [
            'sesame=sesame.sesame:run'
        ]
    }
)
